// Copyright: 	Matra-Datavision 1995
// File:	Select3D_SensitiveSegment.cxx
// Created:	Wed Jan 25 11:27:54 1995
// Author:	Mister rmi
//		<rmi>


#include <Select3D_SensitiveSegment.ixx>
#include <SelectBasics_BasicTool.hxx>
#include <gp_Vec.hxx>
#include <gp_Vec2d.hxx>
#include <Bnd_Box2d.hxx>
#include <gp_Pnt2d.hxx>
#include <gp_Dir2d.hxx>
#include <gp_Lin2d.hxx>
#include <ElCLib.hxx>
#include <TColgp_Array1OfPnt2d.hxx>
#include <Precision.hxx>
#include <SelectBasics_ListIteratorOfListOfBox2d.hxx>

#include <CSLib_Class2d.hxx>

//=====================================================
// Function : Create
// Purpose  :Constructor
//=====================================================


Select3D_SensitiveSegment::
Select3D_SensitiveSegment(const Handle(SelectBasics_EntityOwner)& OwnerId,
			  const gp_Pnt& FirstP,
			  const gp_Pnt& LastP,
			  const Standard_Integer MaxRect):
Select3D_SensitiveEntity(OwnerId),
mymaxrect(MaxRect)
{
  mystart = FirstP;
  myend = LastP;
}

//=====================================================
// Function :
// Purpose  :
//=====================================================
void Select3D_SensitiveSegment
::Project(const Select3D_Projector& aProj)
{
  Select3D_SensitiveEntity::Project(aProj); // to set the field last proj...
  gp_Pnt2d aPoint2dStart;
  gp_Pnt2d aPoint2dEnd;
  
  if(HasLocation()){
    gp_Pnt aStart(mystart.x, mystart.y, mystart.z);
    gp_Pnt aEnd(myend.x, myend.y, myend.z);
    aProj.Project(aStart.Transformed(Location().Transformation()),aPoint2dStart);
    aProj.Project(aEnd.Transformed(Location().Transformation()),aPoint2dEnd);
  }
  else{
    aProj.Project(mystart,aPoint2dStart);
    aProj.Project(myend,aPoint2dEnd);
  }
  myprojstart = aPoint2dStart;
  myprojend = aPoint2dEnd;
}

//=====================================================
// Function : Areas 
// Purpose  :
//=====================================================
void Select3D_SensitiveSegment
::Areas(SelectBasics_ListOfBox2d& theareas)
{
//  gp_Dir2d dy (0.,1.);
  gp_Pnt2d aPStart(myprojstart.x,myprojstart.y);
  if(aPStart.Distance(myprojend)<=Precision::Confusion()){
      Bnd_Box2d curbox;
      curbox.Set(myprojstart);
      theareas.Append(curbox);
    }
  else {
    gp_Vec2d MyVec(myprojstart,myprojend);//,VAxx(gp_Dir2d(0.,1.));
    Standard_Real theangle = Abs(gp_Dir2d(0.,1.).Angle(gp_Vec2d(myprojstart,myprojend)));
    if(theangle>=PI/2.) theangle-=PI/2;
    if(theangle>=PI/12. && theangle <=5*PI/12.)
      {
	TColgp_Array1OfPnt2d BoxPoint (1,mymaxrect+1);
	BoxPoint (1) = myprojstart;
        BoxPoint(mymaxrect+1)=myprojend;
	gp_Vec2d Vtr = MyVec/mymaxrect;
//	for (Standard_Integer i=2;i<=mymaxrect;i++)
	Standard_Integer i;
	for ( i=2;i<=mymaxrect;i++)
	  {BoxPoint (i) = BoxPoint (i-1).Translated(Vtr);}
	for (i=2;i<=mymaxrect+1;i++)
	  { Bnd_Box2d curbox;
	    curbox.Set(BoxPoint(i-1));
	    curbox.Add(BoxPoint(i));
	    theareas.Append(curbox);
	  }
      }
    else
      {
	Bnd_Box2d curbox;
	curbox.Set(myprojstart);
	curbox.Add(myprojend);
	theareas.Append(curbox);
      }
  }
}

//=====================================================
// Function : Matches
// Purpose  :
//=====================================================
Standard_Boolean Select3D_SensitiveSegment
::Matches(const Standard_Real X,
	  const Standard_Real Y,
	  const Standard_Real aTol,
	  Standard_Real&  DMin)
{
  gp_Pnt2d aPStart(myprojstart.x, myprojstart.y);
  if(aPStart.Distance(myprojend)<=Precision::Confusion()){
    DMin = aPStart.Distance(gp_Pnt2d(X,Y));
  }
  else{
    DMin = Abs(gp_Lin2d(myprojstart, gp_Vec2d(myprojstart,myprojend)).Distance(gp_Pnt2d(X,Y)));}
  if(DMin<=aTol) {  Select3D_SensitiveEntity::Matches(X,Y,aTol,DMin);
		    return Standard_True;
		  }
  return Standard_False;
  
}

Standard_Boolean Select3D_SensitiveSegment::
Matches (const Standard_Real XMin,
	 const Standard_Real YMin,
	 const Standard_Real XMax,
	 const Standard_Real YMax,
	 const Standard_Real aTol)
{
  Bnd_Box2d BoundBox;
  BoundBox.Update(XMin-aTol,YMin-aTol,XMax+aTol,YMax+aTol);
  if(BoundBox.IsOut(myprojstart)) return Standard_False;
  if( BoundBox.IsOut(myprojend)) return Standard_False;
  return Standard_True;
}


//=======================================================================
//function : Matches
//purpose  : 
//=======================================================================

Standard_Boolean Select3D_SensitiveSegment::
Matches (const TColgp_Array1OfPnt2d& aPoly,
	 const Bnd_Box2d& aBox,
	 const Standard_Real aTol)
{ 
  Standard_Real Umin,Vmin,Umax,Vmax;
  aBox.Get(Umin,Vmin,Umax,Vmax);
  Standard_Real Tolu,Tolv;
  Tolu = 1e-7;
  Tolv = 1e-7;
  CSLib_Class2d aClassifier2d(aPoly,aTol,aTol,Umin,Vmin,Umax,Vmax);

  Standard_Integer RES = aClassifier2d.SiDans(myprojstart);
  if (RES!=1) return Standard_False;

  RES = aClassifier2d.SiDans(myprojend);
  if (RES!=1) return Standard_False;

  return Standard_True;
}


//=======================================================================
//function : GetConnected
//purpose  : 
//=======================================================================
Handle(Select3D_SensitiveEntity) Select3D_SensitiveSegment::GetConnected(const TopLoc_Location& aLoc)  
{
  Handle(Select3D_SensitiveSegment) NiouEnt = new Select3D_SensitiveSegment(myOwnerId,mystart,myend,mymaxrect);
  if(HasLocation()) NiouEnt->SetLocation(Location());
  NiouEnt->UpdateLocation(aLoc);
  return NiouEnt;
}

//=======================================================================
//function : Dump
//purpose  : 
//=======================================================================

void Select3D_SensitiveSegment::Dump(Standard_OStream& S,const Standard_Boolean FullDump) const
{
  S<<"\tSensitivePoint 3D :"<<endl;
  if(HasLocation())
    S<<"\t\tExisting Location"<<endl;
  S<<"\t\t P1 [ "<<mystart.x<<" , "<<mystart.y <<" , "<<mystart.z <<" ]"<<endl;
  S<<"\t\t P2 [ "<<myend.x<<" , "<<myend.y <<" , "<<myend.z <<" ]"<<endl;
  S<<"\t\t maxrect ="<<mymaxrect<<endl;
  
}
//=======================================================================
//function : ComputeDepth
//purpose  : 
//=======================================================================

Standard_Real Select3D_SensitiveSegment::ComputeDepth(const gp_Lin& EyeLine) const
{
  return (ElCLib::Parameter(EyeLine,mystart)+
          ElCLib::Parameter(EyeLine,myend))/2.;
}
